package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JEditorPane;
import javax.swing.border.LineBorder;

/**
 * Used to display question text and answers.
 *
 * @author Michael Gebauer
 *
 */
public class QuestionPanel extends JEditorPane {

  static final long serialVersionUID = 20050112L;

  public QuestionPanel() {
    setBorder(new LineBorder(Color.BLACK, 2));
    setBackground(Color.WHITE);
    setContentType("text/html");
    setFont(new Font(null, Font.BOLD, 36));
    setEditable(false);
  }

  /**
   * Sets the text for the question panel and adjust the font so that it
   * is as large as possible while fitting into the display.
   *
   * @param s the text to be set
   */
  public void setText(String s) {

    if (s == null) {
      // Clear the display
      super.setText(s);
      return;
    }

    // Find a good text size.

    // System.out.println("setText: " + s);
    // System.out.println("panel size: " + this.getSize());

    // (All this seems to only be a problem for the initial welcome screen.
    // so don't waste more time on it for now.)
    // The width of the box is set, and we wish to determine the font size
    // that will fill the height.  The call to
    // getPreferredScrollableViewportSize isn't right, since it gives a
    // preferred height and width; for a paragraph of text, that height is
    // small (e.g., "width=5591,height=4").  If we have normal text, then
    // SizeTest.java
    // (http://forum.java.sun.com/thread.jspa?threadID=623043&messageID=3531639)
    // shows how to use the LineBreakMeasurer class to compute this.
    // However, if there HTML (as in our case), then that does not work.  I
    // could adjust it to add a break for each HTML construct that does so
    // (<p>, <br>, <li>, etc.), but that seems like a lot of work.

    int size = 36;
    String newText = "<STYLE TYPE=\"text/css\">  div { font-size:"+size+";}</STYLE> <div>"+s+"</div>";
    super.setText(newText);

    // System.out.println("this.getPreferredScrollableViewportSize(): " + this.getPreferredScrollableViewportSize());
    while (this.getHeight() < this.getPreferredScrollableViewportSize().height
           && size>20) {
      // System.out.println("this.getPreferredScrollableViewportSize(): " + this.getPreferredScrollableViewportSize());

      // This re-uses the same size.  Shouldn't size be changed before doing this in the loop?
      // set the font
      newText = "<STYLE TYPE=\"text/css\">  div { font: arial ; font-size:"+size+";}</STYLE> <div>"+s+"</div>";
      super.setText(newText);
      // System.out.println("this.getPreferredScrollableViewportSize(): " + this.getPreferredScrollableViewportSize());

      size -= 4;
    }
  }
}

//  LocalWords:  param css div arial
