package gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * Subclass of DefaultTableCellRenderer which facilitates different colored rows.
 *
 * @author Michael Gebauer
 *
 */
public class ScoreboardRenderer extends DefaultTableCellRenderer {

  static final long serialVersionUID = 20050112L;

  public final static float fontSize = 16;

  /**
   * Identical to the superclass implementation except rows are colored
   * according to rowColors.
   *
   * Any cell containing a string that ends with "*" is colored red.
   * Any cell containing a string that ends with "?" is colored blue.
   */
    public Component getTableCellRendererComponent
        (JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col)
    {
    Component cell = super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,col);
    Team team = (Team) table.getModel().getValueAt(row,0);

    // set the background color
    cell.setBackground(team.getRowColor());

    // if the value ends with a * then show in red
    if (value instanceof String && String.valueOf(value).endsWith("*"))
      cell.setForeground(Color.RED);
    else if (value instanceof String && String.valueOf(value).endsWith("?"))
      cell.setForeground(Color.BLUE);
    else
      cell.setForeground(Color.BLACK);

    // set the font
    cell.setFont(cell.getFont().deriveFont(Font.BOLD,fontSize));
    return cell;
    }
}

//  LocalWords:  DefaultTableCellRenderer rowColors
