package gui;

import java.awt.event.WindowEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;

import scorekeeping.Question;

/**
 * Used to generate a crude summary of all questions and votes in an
 * instance of UpopVote when the instance is terminated.  To use, add this
 * class as a listener.
 *
 * @author Michael Gebauer
 */
public class SummaryGenerator extends java.awt.event.WindowAdapter {

  Collection<Collection<Question>> allQuestions;

  public SummaryGenerator(Collection<Collection<Question>> questions) {
    this.allQuestions = questions;
  }

  /**
   * Generates the summary.
   */
  public void windowClosing(WindowEvent e) {
    StringBuffer summary = new StringBuffer();
    String questionSummary;
    for (Collection<Question> questionSet : allQuestions)
      for (Question question : questionSet) {
        questionSummary = question.writeSummary();
        summary.append(questionSummary+"\n\n");
      }

    File summaryFile = null;
    try {
      summaryFile = new File("upopvote"+File.separator+"summary.txt");
      BufferedWriter writer = new BufferedWriter(new FileWriter(summaryFile));
      writer.write(summary.toString());
      writer.close();
    }
    catch (IOException exception) {
      System.out.println("Failed to generate summary file:  "+summaryFile+"\n"+exception+".  "+exception.getMessage());
    }


    System.exit(0);
  }

}

//  LocalWords:  UpopVote upopvote txt
