package gui;

import ezjcom.JComException;
import ezjcomGenerated._ctlPrsX;
import ezjcomGenerated.ctlPrsX;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Rectangle;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;

import scorekeeping.EZJComUpopVoteConnector;
import scorekeeping.Question;
import scorekeeping.VoteReceiver;
import simpledomparser.SimpleDOMParser;
import simpledomparser.SimpleElement;

public class UpopVote extends JFrame {

  static final long serialVersionUID = 20050112L;
  public static final int MAX_TEAM_SIZE = Team.team0.getIDs().length;
  public static final int MAX_TEAMS = Team.values().length;
  private static boolean haveEZJCom;
  public static final Color BACKGROUNDCOLOR = Color.WHITE;
  public static final boolean VERBOSE = false;

  private QuestionPanel questionPanel;
  private Scoreboard scoreboard;
  private TimerPanel timerPanel;

  public UpopVote(boolean haveEZJCom) {
    super("Groupthink Specification Exercise");
    this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    this.setLayout(new BorderLayout(5,5));
    this.setExtendedState(JFrame.MAXIMIZED_BOTH);

    UpopVote.haveEZJCom = haveEZJCom;

    // create the timer panel
    JPanel timerPanelHolder = new JPanel(new BorderLayout());
    timerPanel = new TimerPanel();
    timerPanelHolder.add(timerPanel,BorderLayout.NORTH);
    getContentPane().add(timerPanelHolder,BorderLayout.EAST);

    // create the question
    questionPanel = new QuestionPanel();
    getContentPane().add(questionPanel, BorderLayout.CENTER);

    // create the scoreboard
    scoreboard = new Scoreboard();
    getContentPane().add(scoreboard,BorderLayout.SOUTH);

    VoteReceiver.setScoreBoardModel(scoreboard.model);

    // turn on the vote receiver
    if (haveEZJCom)
      try {
        ctlPrsX ctl = new ctlPrsX();
        _ctlPrsX _ctl = ctl.get_ctlPrsX();
        _ctl.sInit(new short[1]);
        EZJComUpopVoteConnector receiver = EZJComUpopVoteConnector.createVoteReceiver();
        ctl.addJComEventListener(receiver);
      }
      catch (JComException e1) {
        throw new RuntimeException("Critical error initializing vote receiver!"+e1.getMessage());
      }


    // parse the first question file
    Vector<Question> questionSet1;
    String questionFileName = "upopvote"+File.separator+"round1questions.xml";
    try {
      File questionFile = new File(questionFileName);
      if (! questionFile.exists())
        throw new RuntimeException("File " + questionFileName + " does not exist");
      questionSet1 = parseQuestionsFile(questionFile);
    }
    catch (IOException e) {
      throw new RuntimeException("Problem reading file " + questionFileName + ": " + e.getMessage());
    }

    // parse the second question file
    Vector<Question> questionSet2;
    questionFileName = "upopvote"+File.separator+"round2questions.xml";
    try {
      File questionFile = new File(questionFileName);
      if (! questionFile.exists())
        throw new RuntimeException("File " + questionFileName + " does not exist");
      questionSet2 = parseQuestionsFile(questionFile);
    }
    catch (IOException e) {
      throw new RuntimeException("Problem reading file " + questionFileName + ": " + e.getMessage());
    }

    GameFlowModel timer = new GameFlowModel(this, questionSet1, questionSet2);

    VoteReceiver.setTimer(timer);

    this.setVisible(true);

    List<Collection<Question>> questionSets = new ArrayList<Collection<Question>>(2);
    questionSets.add(questionSet1);
    questionSets.add(questionSet2);

    this.addWindowListener(new SummaryGenerator(questionSets));

    questionPanel.setText(
      "Welcome to the Groupthink Specification Exercise.<br>  " +
      "Please verify that your transmitter works by pressing any number.<br>"   +
      "The last 3 digits of your transmitter ID will appear on the grid below, " +
      "followed by the number you pressed.<br>  " +
      "Later, when answering questions, your transmitter " +
      "ID will be displayed until you have voted.");
  }

  public QuestionPanel getQuestionPanel() {
    return questionPanel;
  }

  public Scoreboard getScoreboard() {
    return scoreboard;
  }

  public TimerPanel getTimerPanel() {
    return timerPanel;
  }

  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
    }
    catch (Exception anyException) {
      // This method throws several exceptions.  If any of them are caught, however
      // simply move on and the default look and feel will automatically be used.
    }

    // check if any of the arguments are true
    boolean parameter = false;
    for (String arg : args)
      if (Boolean.parseBoolean(arg))
        parameter = Boolean.parseBoolean(arg);
    // otherwise run with false as parameter
    UpopVote program = new UpopVote(parameter);
  }

  public static boolean haveEZJCom() {
    return haveEZJCom;
  }

  /**
   * Creates a question set from an XML file.
   */
  public static Vector<Question> parseQuestionsFile(File f)
      throws IOException {
    try {
      // create a reader for the file
      BufferedReader reader = new BufferedReader(new FileReader(f));
      SimpleDOMParser parser = new SimpleDOMParser();
      SimpleElement allQuestions = parser.parse(reader);
      Vector<Question> questions = new Vector<Question>();

      // possibilities: upopvote, question, answer, wrong-answer, time
      // if upopvote recursively add all questions
      if (allQuestions.getTagName().equalsIgnoreCase("upopvote")) {
        SimpleElement simpleElement;
        for (Object someObj : allQuestions.getChildElements()) {
          simpleElement = (SimpleElement) someObj;

          // if question, add all answers / wrong-answers / time if
          // present ... loop trough all possibilities
          if (simpleElement.getTagName().equalsIgnoreCase("question")) {
            // read all the answers
            LinkedHashMap<String,Question.AnswerType> answers = new LinkedHashMap<String,Question.AnswerType>();

            SimpleElement child;
            for (Object o : simpleElement.getChildElements()) {
              child = (SimpleElement) o;


              // is the tag an answer?
              if (child.getTagName().equalsIgnoreCase("answer")) {
                answers.put(child.getText().trim(),Question.AnswerType.correct);
              }

              // is the tag a wrong answer?
              else if (child.getTagName().equalsIgnoreCase(
                  "wrong-answer")) {
                answers.put(child.getText().trim(),Question.AnswerType.wrong);
              }

              // is the tag a strange-answer
              else if (child.getTagName().equalsIgnoreCase(
                  "strange-answer")) {
                answers.put(child.getText().trim(),Question.AnswerType.strange);
              }

              // nothing recognized...
              else
                throw new RuntimeException(
                    "Error parsing SimpleElement.  Unknown tag: "
                        + simpleElement.getTagName());
            }

            // create the question
            String questionText = simpleElement.getText().trim();
            Question newQuestion = new Question(questionText,answers);
            questions.add(newQuestion);
          }
        }
      } else
        throw new RuntimeException(
            "Error parsing SimpleElement.  Unknown tag: "
                + allQuestions.getTagName());

      return questions;
    } catch (FileNotFoundException e1) {
      // TODO:Display a dialog to find a new file
      throw new RuntimeException(
          "File not found / file selection dialog not implemented");
    }
  }

  // Prints each argument on a new line if verbose is turned on
  public static void print(Object...objects) {
    if (VERBOSE)
      for (Object o : objects)
        System.out.println(o);
  }
}

//  LocalWords:  upopvote xml SimpleElement TODO
